/*
 * Pad for Psemu Pro like Emulators
 *
 * By: linuzappz <linuzappz@hotmail.com>
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/cdrom.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

typedef char HWND;

#include "PSEmu Plugin Defs.h"

char *LibName = "LinuzAppz X Windows Pad Driver";

const unsigned char version = 1; // PSEmu 1.x library
const unsigned char revision = VERSION;
const unsigned char build = BUILD;

void LoadConf();
void SaveConf();

short Pad1[16] = {
    XK_space,	// Select
    0,
    0,
    XK_Return,	// Start
    XK_Up,	// Up
    XK_Right,	// Right
    XK_Down,	// Down
    XK_Left,	// Left
    XK_1,	// L2
    XK_3,	// R2
    XK_q,	// L1
    XK_e,	// R1
    XK_w,	// Triangle
    XK_d,	// Circle
    XK_x,	// Cross
    XK_a	// Square
};

short Pad2[16] = {
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0
};

Display *Dsp;
XEvent E;
int PadOpened = 0;
int PadFlags = 0;

char *PSEgetLibName(void) {
    return LibName;
}

unsigned long PSEgetLibType(void) {
    return PSE_LT_PAD;
}

unsigned long PSEgetLibVersion(void) {
    return version<<16|revision<<8|build;
}

long PADinit(long flags) {
    PadFlags|=flags;
    LoadConf();

    return 0;
}

long PADshutdown(void) {
    return 0;
}

long PADopen(unsigned long *Disp) {
    Dsp = (Display *)*Disp;
    PadOpened = 1;
    XAutoRepeatOff(Dsp);
    return 0;
}

long PADclose(void) {
    PadOpened = 0;
    XAutoRepeatOn(Dsp);
    return 0;
}

long PADquery(void) {
    return 3; // both pads
}

typedef struct
{
    unsigned char controllerType;
    unsigned short buttonStatus;
    unsigned char rightJoyX, rightJoyY, leftJoyX, leftJoyY;
    unsigned char moveX, moveY;
    unsigned char reserved[91];
} PadDataS;

short PressedKey;
short ReleasedKey;
unsigned short Pad1Stat = 0xffff;
unsigned short Pad2Stat = 0xffff;

void UpdatePads() {
    int i;

    while (XPending(Dsp)) {
	XNextEvent(Dsp, &E);
	switch (E.type) {
	    case KeyPress:
		PressedKey = XLookupKeysym((XKeyEvent *)&E, 0);
		if (PadFlags & 1) {
		    for (i=0;i<16;i++)
			if (PressedKey == Pad1[i])
			    { PressedKey = 0; Pad1Stat&=~(1<<i); break; }
		    if (i!=16) break;
		}
		if (PadFlags & 2) {
		    for (i=0;i<16;i++)
			if (PressedKey == Pad2[i])
			    { PressedKey = 0; Pad2Stat&=~(1<<i); break; }
		    if (i!=16) break;
		}
		return;
	    case KeyRelease:
		ReleasedKey = XLookupKeysym((XKeyEvent *)&E, 0);
		if (PadFlags & 1)
		    for (i=0;i<16;i++)
			if (ReleasedKey == Pad1[i])
			    { Pad1Stat|=(1<<i); break; }
		if (PadFlags & 2)
		    for (i=0;i<16;i++)
			if (ReleasedKey == Pad2[i])
			    { Pad2Stat|=(1<<i); break; }
		break;
	    case FocusIn:
		XAutoRepeatOff(Dsp);
		break;
	    case FocusOut:
		XAutoRepeatOn(Dsp);
		break;
	}
    }
}
    

long PADreadPort1(PadDataS *pad) {
    UpdatePads();

    pad->buttonStatus = Pad1Stat;
    pad->controllerType = 4; // standard

    return 0;
}

long PADreadPort2(PadDataS *pad) {
    UpdatePads();

    pad->buttonStatus = Pad2Stat;
    pad->controllerType = 4; // standard

    return 0;
}

long PADkeypressed(void) {
    long ret;

    if (PadOpened == 0) return 0;

    UpdatePads();

    if (PressedKey) {
	ret = (unsigned short)PressedKey;
	PressedKey = 0;
	return ret;
    }

    return 0;
}

GtkWidget *ConfDlg;
GtkWidget *Edit;
GtkWidget *KeyLabel;
static char keystring[256];

void OnOk() {
    gtk_widget_hide(ConfDlg);
    gtk_widget_destroy(ConfDlg);
    SaveConf();
    gtk_main_quit();
}

void OnCancel() {
    gtk_widget_hide(ConfDlg);
    gtk_widget_destroy(ConfDlg);
    gtk_main_quit();
}

GtkWidget *CreateButton(GtkWidget *BBox, char *name) {
    GtkWidget *But = gtk_button_new_with_label(name);
    gtk_container_add(GTK_CONTAINER(BBox), But);
    GTK_WIDGET_SET_FLAGS(But, GTK_CAN_DEFAULT);
    gtk_widget_show(But);
    return But;
}

void OnKey(GtkWidget *But, gpointer data) {
    GdkEvent *ev;
    short *key = (short*)gtk_object_get_user_data(GTK_OBJECT(But));
    char *tmp;

    for (;;) {
	ev = gdk_event_get();
	if (ev != NULL) {
	    if  (ev->type == GDK_KEY_PRESS) {
		if (ev->key.keyval != XK_Escape) {
	    	    *key = ev->key.keyval;
		    tmp = XKeysymToString(*key);
		    if (tmp != NULL) {
			strcpy(keystring, "Key: ");
			strcat(keystring, tmp);
			gtk_label_set(GTK_LABEL(KeyLabel), keystring);
		    } else {
			strcpy(keystring, "Key: (Unknown name key)");
			gtk_label_set(GTK_LABEL(KeyLabel), keystring);
		    }
		}
		else {
		    strcpy(keystring, "Key: Can't set Escape key");
		    gtk_label_set(GTK_LABEL(KeyLabel), keystring);
		}
		return;
	    }
	}
    }
}

#define CreateKeyBut(name, key) \
    Ok = CreateButton(BBox, name); \
    gtk_signal_connect(GTK_OBJECT(Ok), "clicked", GTK_SIGNAL_FUNC(OnKey), NULL); \
    gtk_object_set_user_data(GTK_OBJECT(Ok), (char *)&key);

long PADconfigure(void) {
    GtkWidget *Label;
    GtkWidget *Ok,*Cancel;
    GtkWidget *Box,*Box1,*BBox;

    LoadConf();

    ConfDlg = gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_widget_set_usize(ConfDlg, 460, 320);
    gtk_window_set_title(GTK_WINDOW(ConfDlg), "PAD Config Dialog");
    gtk_window_set_position(GTK_WINDOW(ConfDlg), GTK_WIN_POS_CENTER);
    gtk_container_set_border_width(GTK_CONTAINER(ConfDlg), 10);

    Box = gtk_vbox_new(0, 0);
    gtk_container_add(GTK_CONTAINER(ConfDlg), Box);
    gtk_widget_show(Box);

    Box1 = gtk_hbox_new(0, 0);
    gtk_box_pack_start(GTK_BOX(Box), Box1, FALSE, FALSE, 0);
    gtk_widget_show(Box1);

    Label = gtk_label_new("Pad1:");
    gtk_box_pack_start(GTK_BOX(Box1), Label, FALSE, FALSE, 0);
    gtk_widget_show(Label);

    BBox = gtk_vbutton_box_new();
    gtk_box_pack_start(GTK_BOX(Box1), BBox, FALSE, FALSE, 0);
    gtk_vbutton_box_set_spacing_default(0);
    gtk_widget_show(BBox);

    CreateKeyBut("Select", Pad1[0]);
    CreateKeyBut("Start", Pad1[3]);
    CreateKeyBut("Up", Pad1[4]);
    CreateKeyBut("Right", Pad1[5]);
    CreateKeyBut("Down", Pad1[6]);
    CreateKeyBut("Left", Pad1[7]);

    BBox = gtk_vbutton_box_new();
    gtk_box_pack_start(GTK_BOX(Box1), BBox, FALSE, FALSE, 0);
    gtk_vbutton_box_set_spacing_default(0);
    gtk_widget_show(BBox);

    CreateKeyBut("L2", Pad1[8]);
    CreateKeyBut("R2", Pad1[9]);
    CreateKeyBut("L1", Pad1[10]);
    CreateKeyBut("R1", Pad1[11]);
    CreateKeyBut("Triangle", Pad1[12]);
    CreateKeyBut("Circle", Pad1[13]);
    CreateKeyBut("Cross", Pad1[14]);
    CreateKeyBut("Square", Pad1[15]);

    Label = gtk_label_new("Pad2:");
    gtk_box_pack_start(GTK_BOX(Box1), Label, FALSE, FALSE, 10);
    gtk_widget_show(Label);

    BBox = gtk_vbutton_box_new();
    gtk_box_pack_start(GTK_BOX(Box1), BBox, FALSE, FALSE, 0);
    gtk_vbutton_box_set_spacing_default(0);
    gtk_widget_show(BBox);

    CreateKeyBut("Select", Pad2[0]);
    CreateKeyBut("Start", Pad2[3]);
    CreateKeyBut("Up", Pad2[4]);
    CreateKeyBut("Right", Pad2[5]);
    CreateKeyBut("Down", Pad2[6]);
    CreateKeyBut("Left", Pad2[7]);

    BBox = gtk_vbutton_box_new();
    gtk_box_pack_start(GTK_BOX(Box1), BBox, FALSE, FALSE, 0);
    gtk_vbutton_box_set_spacing_default(0);
    gtk_widget_show(BBox);

    CreateKeyBut("L2", Pad2[8]);
    CreateKeyBut("R2", Pad2[9]);
    CreateKeyBut("L1", Pad2[10]);
    CreateKeyBut("R1", Pad2[11]);
    CreateKeyBut("Triangle", Pad2[12]);
    CreateKeyBut("Circle", Pad2[13]);
    CreateKeyBut("Cross", Pad2[14]);
    CreateKeyBut("Square", Pad2[15]);

    KeyLabel = gtk_label_new("Key:");
    gtk_box_pack_start(GTK_BOX(Box), KeyLabel, FALSE, FALSE, 0);
    gtk_widget_show(KeyLabel);

    BBox = gtk_hbutton_box_new();
    gtk_box_pack_start(GTK_BOX(Box), BBox, FALSE, FALSE, 0);
    gtk_widget_show(BBox);

    Ok = CreateButton(BBox, "Ok");
    gtk_signal_connect(GTK_OBJECT(Ok), "clicked", GTK_SIGNAL_FUNC(OnOk), NULL);

    Cancel = CreateButton(BBox, "Cancel");
    gtk_signal_connect(GTK_OBJECT(Cancel), "clicked", GTK_SIGNAL_FUNC(OnCancel), NULL);

    gtk_widget_show(ConfDlg);

    gtk_main();

    return 0;
}

GtkWidget *AboutDlg;

void OnAboutOk() {
    gtk_widget_hide(AboutDlg);
    gtk_widget_destroy(AboutDlg);
    gtk_main_quit();
}

void PADabout(void) {
    GtkWidget *Label;
    GtkWidget *Ok;
    GtkWidget *Box,*BBox;
    char AboutText[255];

    sprintf (AboutText,"%s %d.%d\n",LibName, revision, build);

    AboutDlg = gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_widget_set_usize(AboutDlg, 260, 80);
    gtk_window_set_title(GTK_WINDOW(AboutDlg), "PAD About Dialog");
    gtk_window_set_position(GTK_WINDOW(AboutDlg), GTK_WIN_POS_CENTER);
    gtk_container_set_border_width(GTK_CONTAINER(AboutDlg), 10);

    Box = gtk_vbox_new(0, 0);
    gtk_container_add(GTK_CONTAINER(AboutDlg), Box);
    gtk_widget_show(Box);

    Label = gtk_label_new(AboutText);
    gtk_box_pack_start(GTK_BOX(Box), Label, FALSE, FALSE, 0);
    gtk_widget_show(Label);

    BBox = gtk_hbutton_box_new();
    gtk_box_pack_start(GTK_BOX(Box), BBox, FALSE, FALSE, 0);
    gtk_widget_show(BBox);

    Ok = gtk_button_new_with_label("Ok");
    gtk_signal_connect(GTK_OBJECT(Ok), "clicked", GTK_SIGNAL_FUNC(OnAboutOk), NULL);
    gtk_container_add(GTK_CONTAINER(BBox), Ok);
    GTK_WIDGET_SET_FLAGS(Ok, GTK_CAN_DEFAULT);
    gtk_widget_show(Ok);

    gtk_widget_show(AboutDlg);

    gtk_main();
}

long PADtest(void) {
    return 0;
}

void LoadConf() {
    FILE *f;
    char cfg[255];

    sprintf (cfg,"%s/padXwin.cfg",getenv("HOME"));
    f = fopen(cfg,"r");
    if (f == NULL) return;
    fread(Pad1, 2, 16, f);
    fread(Pad2, 2, 16, f);
    fclose(f);
}

void SaveConf() {
    FILE *f;
    char cfg[255];

    sprintf (cfg,"%s/padXwin.cfg",getenv("HOME"));
    f = fopen(cfg,"w");
    if (f == NULL) return;
    fwrite(Pad1, 2, 16, f);
    fwrite(Pad2, 2, 16, f);
    fclose(f);
}
